# -*- coding: utf-8 -*-
# Copyright 2010 Intevation GmbH, Germany, <info@intevation.de>
# Authors:
# Bernhard Herzog <bernhard.herzog@intevation.de>

"""Structure instance trees that match the meta structure trees."""

from formed.meta.data import RepeatNode


class StructureInstanceNode(object):

    """Node of the structure instance tree.

    Public read-only instance variables:

       uuid -- The uuid of the node

       id -- The database id of the node.  None if the node is not in
             the database

       meta_node -- The corresponding meta structure node.
    """

    def __init__(self, meta_node, id, uuid=None, leaves=None,
                 repeat_groups=None):
        self.meta_node = meta_node
        self.id = id
        self.uuid = uuid
        if leaves is None:
            leaves = dict()
        self.leaves = leaves
        if repeat_groups is None:
            repeat_groups = dict()
        self.repeat_groups = repeat_groups

    def get(self, name):
        local_name = name[0]
        restname = name[1:]

        if isinstance(local_name, tuple):
            item = self.get_repeat_group(*local_name)
            if restname:
                item = item.get(restname)
        else:
            meta_leaf = self.meta_node.leaves.get(local_name)
            if meta_leaf is None:
                raise ValueError("Name %r is not known as a leaf" % local_name)
            item = self.get_leaf(local_name)
        return item

    def set_value(self, name, value):
        local_name = name[0]
        restname = name[1:]

        if isinstance(local_name, tuple):
            item = self.get_repeat_group(*local_name)
            if restname:
                item.set_value(restname, value)
        else:
            meta_leaf = self.meta_node.leaves.get(local_name)
            if meta_leaf is None:
                raise ValueError("Name %r is not known as a leaf" % local_name)
            self.set_leaf(local_name, value)

    def get_leaf(self, meta_name):
        return self.leaves[meta_name]

    def set_leaf(self, meta_name, value):
        self.leaves[meta_name] = value

    def get_repeat_groups(self, rg_name):
        if rg_name not in self.meta_node.repeat_groups:
            raise ValueError("Node %r has no repeat group child %r"
                             % (self.meta_node.name, rg_name))
        return self.repeat_groups.setdefault(rg_name, dict())

    def get_repeat_group(self, rg_name, rg_id):
        return self.get_repeat_groups(rg_name)[rg_id]


def pprint_structure_instance_node(node, indent=""):
    """Recursively pretty-print a structure instance node to stdout"""
    print ("%s%s:%s:%s: %d leaves, %d repeat groups"
           % (indent, node.meta_node.name, node.id, node.uuid,
              len(node.leaves), len(node.repeat_groups)))
    indent = "    " + indent
    for key, item in sorted(node.leaves.items()):
        print "%s%s: %r" % (indent, key, item)
    for name, rgs in sorted(node.repeat_groups.items()):
        for key, item in sorted(rgs.items()):
            pprint_structure_instance_node(item, indent)


class StructureInstanceTree(object):

    """Structure instance tree.

    Public read-only instance variables:

      root -- The root structure instance node

      meta_structure -- The corresponding structure meta tree

      uuid -- The uuid of the tree.  Identical to the uuid of the root
              node.

      id -- The database id of the instance.  Identical to the id of the
            root node.
    """

    def __init__(self, meta_structure, root=None, id=None, uuid=None):
        self.meta_structure = meta_structure
        if root is not None:
            self.root = root
        else:
            self.root = self.instantiate_root(meta_structure.root, id, uuid)

    @property
    def uuid(self):
        return self.root.uuid

    @property
    def id(self):
        return self.root.id

    def instantiate_root(self, meta_root, id, uuid):
        return StructureInstanceNode(meta_root, id, uuid=uuid)

    def get(self, name):
        """Return the value of the item indicated by name.
        The argument name can be of two forms: it may be a string using
        the legacy syntax used by formed.instance.tree, or a new item
        specification.

        If the name is a string containing at least one colon (':'), it
        is assumed to be a legacy name.  Legacy names are converted to
        item specifications and then treated like those.

        If the name is not a legacy name, it's an item specification.
        Item specifications have form (rgid1, rgid2, ..., itemname).
        The last part of that tuple is the name of the item as used in
        the meta tree.  The rgidN values are integers specifying the ids
        of the repeat groups that contain the instance of the item.  The
        meta structure defines how many repeat groups there are in the
        hierarchy above the item, so the number of repeat group ids
        required is always known.

        If itemname ist the name of a repeat group, there are two cases
        to consider, though.  The item specification can refer to all
        instances of the repeat group or to a particular instance.  If
        the the number of repeat group ids given in the item
        specification is equal to the number of the repeat groups above
        the item named by itemname, the specification refers to all
        instances.  If there's one more repeat group id, it is
        interpreted as the id of a particular instance of the repeat
        group named by itemname.

        Another special case is items that are not contained in repeat
        groups.  In this case the item name can be given directly as a
        string.  It does not have to be wrapped in a single-element
        tuple.
        """
        return self.root.get(self.full_name(name))

    def set_value(self, name, value):
        self.root.set_value(self.full_name(name), value)

    def full_name(self, name):
        # name is either simply a string with a field name, or a tuple
        # (rgid1, ..., rgidN, fieldname)
        if isinstance(name, basestring):
            if ":" in name:
                name = self.parse_legacy_name(name)
            else:
                name = (name,)

        path = self.meta_structure.structure_path(name[-1])
        num_ids = len(name) - 1
        path_len = len(path)
        if isinstance(path[-1], RepeatNode):
            if num_ids not in (path_len, path_len - 1):
                raise ValueError("The full name for repeat group %r must have"
                                 " either %d or %d ids."
                                 % (name, path_len - 1, path_len))
        else:
            if num_ids != path_len - 1:
                raise ValueError("The full name for item %r must have exactly "
                                 " %d ids." % (name, path_len - 1))
        full_name = []
        for i in range(num_ids):
            full_name.append((path[i].getName(), name[i]))
        if num_ids < path_len:
            full_name.append(name[-1])
        return full_name

    def parse_legacy_name(self, legacy_name):
        """Convert a legacy instance item name into the new item specification.
        The legacy name is a string consisting of colon-separated
        fields.  The first field is the item's meta name.  The rest of
        the fields are the ids of the repeat groups containing the item,
        starting with the one immediately containing the instance item,
        followed by the id of the repeat group containing that repeat
        group, and so on.  The master table itself is treated as a
        repeat group in this context, so the last repeat group id is
        always the case id.
        """
        assert ":" in legacy_name
        parts = legacy_name.split(":")
        parts.reverse()
        for i in range(len(parts) - 1):
            parts[i] = int(parts[i])
        if parts[0] != self.root.id:
            raise ValueError("Legacy name must contain the case id")
        return tuple(parts[1:])

    def get_all(self, name):
        """Return all values for the leaf named by name.
        The return value is a sequence of tuples of the form (NAME,
        VALUE) where NAME is a name usable for the get method to
        retrieve the value and VALUE is the current value of the item.
        """
        def generate_items(node, path):
            child_meta = path[0]
            child_name = child_meta.getName()
            if isinstance(child_meta, RepeatNode):
                for rg in node.get_repeat_groups(child_name).values():
                    for partial_id, value in generate_items(rg, path[1:]):
                        yield ((rg.id,) + partial_id, value)
            else:
                yield ((child_name,), node.get((child_name,)))

        path = self.meta_structure.structure_path(name)

        return list(generate_items(self.root, path))
