# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3. 
# See LICENSE comming with the source of 'mpuls'
# for details.
#
# author: Torsten Irländer <torsten.irlaender@intevation.de>

import os
import traceback
from lxml import etree
from StringIO import StringIO

XML_VERSION = u"""<!--
The following result of the evaluation was generated with the following
components in the named versions.

prequery:   %s
libmpuls:   %s
evalconfig: %s
datadase:   %s
datadump:   %s
-->
"""
XML_SET    = u"""<evaluationset id="%s"><name>%s</name><desc>%s</desc><agency>%s</agency><start_date>%s</start_date><end_date>%s</end_date><requirement><phase>%s</phase>%s</requirement><num><total>%s</total><valid>%s</valid><considered>%s</considered></num>%s</evaluationset>"""
XML_EVAL   = u"""<evaluation id="%s" name="%s"><desc>%s</desc><requirement><phase>%s</phase>%s</requirement><num><total>%s</total><valid>%s</valid><considered>%s</considered></num>%s</evaluation>"""
XML_TABLE  = u"""<table xlabel="%s" ylabel="%s"><headers>%s</headers><rows>%s</rows></table>"""
XML_HEADER = u"""<header>%s</header>"""
XML_ROW    = u"""<row label="%s">%s</row>"""
XML_COL    = u"""<col>%s</col>"""

class EvaluationExport:
    '''
    Export handler for evaluation results

    The result of a L{Evaluation} can be exported in various formats. Currently
    the following formats are supported

        1. XML @see: L{EvaluationExportXML}
        2. CSV @see: L{EvaluationExportCSV}

    On default the export is done by transforming the resultset of the
    evaluation to XML first. This XML can then be transformed by using XSLT
    into to target format.

    DEFAULT XML-FORMAT
    ==================
    See L{EvaluationExportXML}

    WRITING OWN EXPORTERS
    =====================
    New exported can and should be createted by inheritation of
    EvaluationExport and overwriting the 'transform'-function
    '''

    def __init__(self, style=None, show_percent=False):
        '''
        @param style: Path of the XSLT-style which will be loaded to transform
        the XML into the Export format.
        @type style: String
        @param show_percent: Calculate percent based on total for each row and
        display it. Only supported bei HTML export yet.
        @type style: Boolean
        '''
        self.style = style 
        self.show_percent = show_percent

    def _get_required_validity(self, v):
        out = []
        for i in v:
            out.append('<validity>%s</validity>' % i)
        return "\n".join(out)

    def _toXML(self, evalset):
        xml = []
        evals = evalset.evals
        xml.append('<?xml version="1.0"?>')
        xml.append(XML_VERSION 
            % (evalset.v_prequery,
               evalset.v_libmpuls,
               evalset.v_evalconfig,
               evalset.v_database,
               evalset.v_dataset))
        # Set
        setid       = evalset.id
        setname     = evalset.name
        setdesc     = evalset.desc
        agency      = evalset.agency
        sdate       = evalset.sdate
        edate       = evalset.edate
        setphase    = evalset.rphase
        setvalidity = evalset.rvalidity
        settotal    = evalset.num_total
        setvalid    = evalset.num_valid
        setcons     = evalset.num_considered
        # Evaluation
        evalout = []
        for eval in evals:
            rows       = []
            id         = eval.id
            name       = eval.name
            desc       = eval.desc
            eval_phase = eval.rphase
            eval_total = eval.num_total
            eval_valid = eval.num_valid
            eval_cons  = eval.num_considered
            headers = []
            table   = []
            for result in eval.get_result():
                xlabel = result.xlabel
                ylabel = result.ylabel
                rows = []
                for rrow in result.result:
                    for y, x in rrow.iteritems():
                        y = y.replace('"', "'") # Replace double quotes as they result in problems later in exports.
                        cols    = []
                        headers = []
                        for label, value in x:
                            cols.append(XML_COL % (unicode(str(value), 'UTF-8')))
                            headers.append(XML_HEADER % (unicode(label, 'UTF-8')))
                        try:
                            rows.append(XML_ROW % (y, "\n".join(cols)))
                        except: 
                            rows.append(XML_ROW % (unicode(y, 'UTF-8'), "\n".join(cols)))
                table.append(XML_TABLE % (xlabel, ylabel, "\n".join(headers), "\n".join(rows)))
            evalout.append(XML_EVAL % (
                id, 
                name, 
                desc, 
                eval_phase,
                self._get_required_validity(eval.rvalidity),
                eval_total, 
                eval_valid, 
                eval_cons, 
                "\n".join(table)))
        xml.append(XML_SET % (
            setid, 
            setname, 
            setdesc,
            agency,
            sdate, 
            edate, 
            setphase,
            self._get_required_validity(setvalidity),
            settotal, 
            setvalid,  
            setcons,  
            "\n".join(evalout)))
        return "\n".join(xml) 

    def transform(self, xml):
        '''
        Returns the transformed evaluation result.
        Overwrte this function if you found to implement your own exporthandler.

        @param xml: XML-data
        @type xml: String
        @return: Transformed evaluation
        @rtype: Depends see contrete ExportHandler implementation
        '''
        result_doc = xml
        if self.style:
            try:
                path = self.style
                xslt       = open(path, 'r')
                xslt_doc   = etree.parse(xslt)
                transform  = etree.XSLT(xslt_doc)
                xml_doc    = etree.parse(StringIO(xml))
                result_doc = transform(xml_doc)
                xslt.close()
                # Find out how to get the payload even if the result doc does
                # not have a rootnode. e-g CSV-Export
                if result_doc.getroot() is not None:
                    return etree.tostring(result_doc.getroot(), pretty_print=True)
                else:
                    return result_doc 
            except StandardError, err:
                print 'Error while transforming error: %s' % err
        else:
            result_doc = xml 
            #log.debug('No path to XSL-file defined. Returning untransformed xml')
        return result_doc 

    def export(self, eval):
        return self.transform(self._toXML(eval))

class EvaluationExportXML(EvaluationExport):
    '''
    Export for default mpuls evaluation XML

    XML-FORMAT
    ==========
    The following XML shows the default structure the exportes XML::

        <evaluationset id="">
          <desc></desc>
          <name></name>
          <agency></agency>
          <start_date></start_date>
          <end_date></end_date>
          <evaluation id="" name="">
            <desc></desc>
            <table xlabel="" ylabel="">
              <headers>
                <header>
                </header>
              </header>
              <rows>
                <row label="">
                  <col></col>
                </row>
              </rows>
            </table>
            <table xlabel="" ylabel="">
              <headers>
                <header>
                </header>
              </header>
              <rows>
                <row label="">
                  <col></col>
                </row>
              </rows>
            </table>
          </evaluation>
        </evaluationset>
    '''

    def transform(self, xml):
        return xml

class EvaluationExportCSV(EvaluationExport):

    def transform(self, xml):
        '''
        Returns the transformed evaluation result as CSV.

        @param xml: XML-data
        @type xml: String
        @return: HTML transformed evaluation
        @rtype: String 
        '''
        result_doc = xml
        try:
            xml_doc    = etree.parse(StringIO(xml))
            result_doc = self._buildCSV(xml_doc)
        except StandardError, err:
            print 'Error while transforming error: %s' % err
            print traceback.print_exc()
        return result_doc 

    def export(self, eval):
        return self.transform(self._toXML(eval))

    #def _get_max_length(eval):
    #    xlabelgroupings = []
    #    length = 0
    #    for num, table in enumerate(xml.findall('table')):
    #        xlabel = table.attrib['xlabel']
    #        if xlabel not in xlabelgroupings:
    #            xlabelgroupings.append(xlabel)
    #            length += len(table.findall('headers/header'))
    #    return length

    def _build_requirments(self, pha, val, tot, tov, toc):
        out = []
        out.append(u',,"%s: Von insgesamt %s Fällen, erfüllen %s die grundlegenden Vorbedingungen für eine Auswertung (Phase mindest. %s und Validitätsstufen %s). Von diesen %s Fällen erfüllen %s die vom Benutzer gesetzten Parameter wie z.B Datumsgrenzen und bilden somit die Grundmenge für die Auswertung."' % ('Grundmenge', tot, tov, pha, val, tov, toc))
        return out

    def _buildCSV(self, xml):
        out = []
        out.append(',,"Ergebnis der Auswertung"')
        out.append('\n')
        out.append(',,"Name: %s"' % xml.find('name').text)
        out.append('\n')
        out.append(',,"Beschreibung: %s"' % xml.find('name').text)
        out.append('\n')
        out.append(',,"Startdatum: %s"' % xml.find('start_date').text)
        out.append('\n')
        out.append(',,"Enddatum: %s"' % xml.find('end_date').text)
        out.append('\n')
        pha = xml.find('requirement/phase').text or "-/-" 
        val = ",".join([v.text for v in xml.findall('requirement/validity')]) or "-/-" 
        tot = xml.find('num/total').text 
        tov = xml.find('num/valid').text 
        toc = xml.find('num/considered').text 
        out.extend(self._build_requirments(pha, val, tot, tov, toc))
        out.append('\n')
        out.append('\n')

        root = xml.getroot()
        for eval in root.findall('evaluation'):
            out.append(',,"%s (%s)"' % (eval.attrib['name'], eval.attrib['id']))
            out.append('\n')
            out.append(',,"%s"' % eval.find('desc').text)
            out.append('\n')
            out.extend(self._buildEval(eval))
            out.append('\n')
            pha = eval.find('requirement/phase').text or "-/-" 
            val = ",".join([v.text for v in eval.findall('requirement/validity')]) or "-/-" 
            tot = eval.find('num/total').text 
            tov = eval.find('num/valid').text
            toc = eval.find('num/considered').text
            out.extend(self._build_requirments(pha, val, tot, tov, toc))
            out.append('\n')
        return "".join(out)

    def _buildEval(self, xml):
        out = []
        # Build table header grouping
        xlabelgroupings = []
        xlabels         = []
        ylabelgroupings = []
        ylabels         = []
        for num, table in enumerate(xml.findall('table')):
            xlabel = table.attrib['xlabel']
            ylabel = table.attrib['ylabel']
            if xlabel not in xlabelgroupings:
                xlabelgroupings.append(xlabel)
                if num == 0:
                    out.append(',,')
                out.append('"%s"%s' % (xlabel, ","*len(table.findall('headers/header'))))
                xlabels.extend(table.findall('headers/header'))
            if ylabel not in ylabelgroupings:
                ylabelgroupings.append(ylabel)
        out.append('\n')

        # Build xlabels 
        out.append(',,')
        out.append(",".join('"%s"' % label.text for label in xlabels))
        out.append('\n')

        # Build datarows
        for ylabel in ylabelgroupings:
            data = []
            for xlabel in xlabelgroupings:
                table = xml.xpath('table[@xlabel="%s"][@ylabel="%s"]' % (xlabel, ylabel))
                data.append(table)
            out.extend(self._buildRows(data))
        out.append('\n')
        return out

    def _buildRows(self, tables):
        out   = []
        rows = tables[0][0].findall('rows/row')
        for n in range(len(rows)):
            row = []
            for m in range(len(tables)):
                r = tables[m][0].findall('rows/row')[n]
                if m == 0:
                    if n == 0:
                        row.append('"%s"' % (tables[0][0].attrib['ylabel']))
                    row.append(',"%s"' % r.attrib['label'])
                row.extend(self._buildCols(r))
            out.extend(row)
            out.append('\n')
        return out

    def _buildCols(self, row):
        out = []
        out.append(",")
        out.append(",".join('"%s"' % col.text for col in row.findall('col')))
        return out

class EvaluationExportHTML(EvaluationExport):

    def transform(self, xml):
        '''
        Returns the transformed evaluation result as HTML table.

        @param xml: XML-data
        @type xml: String
        @return: HTML transformed evaluation
        @rtype: String 
        '''
        result_doc = xml
        try:
            print type(xml)
            xml_doc    = etree.parse(StringIO(xml))
            result_doc = self._buildHtml(xml_doc)
        except StandardError, err:
            print 'Error while transforming error: %s' % err
            print traceback.print_exc()
        return result_doc 

    def export(self, eval):
        return self.transform(self._toXML(eval))

    def _build_requirments(self, pha, val, tot, tov, toc):
        out = []
        out.append(u'<strong>%s</strong>: Von insgesamt %s Fällen, erfüllen %s die grundlegenden Vorbedingungen für eine Auswertung (Phase mindest. %s und Validitätsstufen %s). Von diesen %s Fällen erfüllen <span class="ok">%s</span> die vom Benutzer gesetzten Parameter wie z.B Datumsgrenzen und bilden somit die Grundmenge für die Auswertung.' % ('Grundmenge', tot, tov, pha, val, tov, toc))
        return out

    def _buildHtml(self, xml):
        out = []
        out.append('<div class="evaluation">')
        out.append('<div class="header">')
        out.append('<h1>Ergebnis der Auswertung</h1>')
        out.append('</div>')
        out.append('<div>')
        out.append('<p>')
        out.append('<strong>Name</strong>: %s<br>' % xml.find('name').text)
        out.append('<strong>Beschreibung</strong>: %s<br>' % xml.find('desc').text)
        out.append('Startdatum: %s<br>' % xml.find('start_date').text)
        out.append('Enddatum: %s<br>' % xml.find('end_date').text)
        out.append('</p>')
        pha = xml.find('requirement/phase').text or "-/-" 
        val = ",".join([v.text for v in xml.findall('requirement/validity')]) or "-/-" 
        tot = xml.find('num/total').text 
        tov = xml.find('num/valid').text 
        toc = xml.find('num/considered').text 
        out.append('<p class="note">')
        out.extend(self._build_requirments(pha, val, tot, tov, toc))
        out.append('</p>')
        out.append('</div>')

        root = xml.getroot()
        for eval in root.findall('evaluation'):
            out.append('<div class="widget container">')
            out.append('<h2>%s (%s)</h2>' % (eval.attrib['name'], eval.attrib['id']))
            out.append('<p>%s</p>' % eval.find('desc').text)
            out.extend(self._buildEval(eval))
            pha = eval.find('requirement/phase').text or "-/-" 
            val = ",".join([v.text for v in eval.findall('requirement/validity')]) or "-/-" 
            tot = eval.find('num/total').text 
            tov = eval.find('num/valid').text 
            toc = eval.find('num/considered').text 
            out.append('<p class="note">')
            out.extend(self._build_requirments(pha, val, tot, tov, toc))
            out.append('</p>')
            out.append('</div>')
        out.append('</div>')
        return "".join(out)

    def _buildEval(self, xml):
        out = []
        out.append('<table>')
        # Build table header grouping
        xlabelgroupings = []
        xlabels         = []
        ylabelgroupings = []
        ylabels         = []
        out.append('<tr>')
        for num, table in enumerate(xml.findall('table')):
            xlabel = table.attrib['xlabel']
            ylabel = table.attrib['ylabel']
            if xlabel not in xlabelgroupings:
                xlabelgroupings.append(xlabel)
                if num == 0:
                    out.append('<th colspan="2" rowspan="2"></th>')
                out.append('<th colspan="%s">%s</th>' % (len(table.findall('headers/header')), xlabel))
                xlabels.extend(table.findall('headers/header'))
            if ylabel not in ylabelgroupings:
                ylabelgroupings.append(ylabel)
        out.append('</tr>')

        # Build xlabels 
        out.append('<tr>')
        #out.append('<th colspan="2"></th>') # disbled -> see rowspan above
        for label in xlabels:
            out.append('<th>%s</th>' % label.text)
        out.append('</tr>')

        # Build datarows
        for ylabel in ylabelgroupings:
            data = []
            for xlabel in xlabelgroupings:
                table = xml.xpath('table[@xlabel="%s"][@ylabel="%s"]' % (xlabel, ylabel))
                data.append(table)
            out.extend(self._buildRows(data))
        out.append('</table>')
        return out

    def _buildRows(self, tables):
        out   = []
        rows = tables[0][0].findall('rows/row')
        for n in range(len(rows)):
            out.append('<tr>')
            row = []
            for m in range(len(tables)):
                r = tables[m][0].findall('rows/row')[n]
                if m == 0:
                    # Total amount of cases in a single row.  Note this is the
                    # total amount of the first xtable.
                    cols = r.findall('col')
                    total = float(cols[-1].text) # total should be the last col
                    if n == 0:
                        row.append('<td rowspan="%s">%s</td>' % (len(rows), tables[0][0].attrib['ylabel']))
                    row.append("<td>%s</td>" % r.attrib['label'])
                row.extend(self._buildCols(r, total))
            out.extend(row)
            out.append('</tr>')
        return out

    def _buildCols(self, row, total=None):
        total = total
        out = []
        cols = row.findall('col')
        for col in cols:
            cur  = float(col.text)
            try:
                perc = round(cur/total, 4)
            except:
                perc = 1
            if self.show_percent:
                out.append('<td class="num">%s&nbsp;<span class="note">(%s%%)</span></td>' % (int(cur), perc*100))
            else:
                out.append('<td class="num">%s</td>' % (int(cur)))
        return out

class EvaluationExportAdele(EvaluationExport):
    '''
    Export Adele spcific XML. 
    '''

    def __init__(self, style=None):
        EvaluationExport.__init__(self, os.path.join(os.path.dirname(__file__), 'xslt', 'adele.xsl'))

#class MyEvaluationExport(EvaluationExport):
#
#    def set_renderer(self, renderer):
#        self.renderer = renderer
#
#    def transform(self, xml):
#        result_doc = xml
#        try:
#            xml_doc    = etree.parse(StringIO(xml))
#            result_doc = self._build(xml_doc)
#        except StandardError, err:
#            print 'Error while transforming error: %s' % err
#            print traceback.print_exc()
#        return result_doc
#
#    def _build(self, dom):
#        out = []
#        out.extend(_build_evalsetheader(dom))
#        root = dom.getroot()
#        for eval in root.findall('evaluation'):
#            out.extend(_build_eval(dom))
#        return "".join(out)
#
#    def _build_evalsetheader(self, dom):
#        name  = dom.find('name').text
#        desc  = dom.find('desc').text
#        sdate = dom.find('start_date').text
#        edate = dom.find('end_date').text


#class EvaluationExportCSV(EvaluationExport):
#
#    '''
#    @note: Currently the CSV Export only works for evaluationsets which contain only one single evaluation.
#    @todo: Enhance XSL to calculate the max amount of cols in the whole set and fill missing ";" in the evaluations.
#    '''
#    
#    def __init__(self, style=None):
#        EvaluationExport.__init__(self, os.path.join(os.path.dirname(__file__), 'xslt', 'csv.xsl'))
#

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
