import os
import logging
import formencode
import pylons

from pylons import g, c

from libmpuls.evaluation.config import EvaluationConfig
from libmpuls.evaluation.evaluation import EvaluationSet
from libmpuls.evaluation.export import EvaluationExportXML, \
     EvaluationExportCSV, EvaluationExportHTML

from mpulsweb.lib.translation import _
from mpulsweb.lib.base import BaseController, render, request, response, session
from mpulsweb.lib.db import db
from mpulsweb.lib.validators import EvaluationFormValidator
from mpulsweb.model.phase import PhaseFactory, get_phase_successors


log = logging.getLogger(__name__)


def get_configfile(id):
    for enabled_eval in g.mpuls_config.get('evaluations', 'enabled'):
        if enabled_eval.get('id') == str(id):
            filename = enabled_eval.get('config')
            for d in pylons.config.get('pylons.paths').get('static_files'):
                path = os.path.join(d, 'xml', filename)
                if os.path.exists(path):
                    return path
    return None

def get_phase_dates(id):
    '''Helper function to get start and end dates of phases ready to use in
    SQL-Statements for evaluation based on the configuration. Note that
    enddates will have a coalesce clause.'''
    sdate, edate = None, None
    phaselist = PhaseFactory().build()
    phase = None
    for p in phaselist:
        if p.id == id:
            phase = p
    if phase:
        start, end = phase.start, phase.end
        sdate = start.datefield
        edate = "coalesce(%s, now())" % end.datefield
    log.debug('sdate: %s, edate: %s' % (sdate, edate))
    return sdate, edate


def get_search_options(soptions=None, id=None, selected_ids=()):
    options = {}
    options['id'] = id
    options["selected_ids"] = " ".join(map(unicode, selected_ids))
    options['typelist'] = c.evalconfig.get_evaluations()

    # set default evaluation options.
    options['phase'] = g.mpuls_config.get('evaluations', 'default-phases')
    options['type_ending'] = g.mpuls_config.get('evaluations', 'default-ending')
    options['start_date'] = g.mpuls_config.get_date('search',
                                                    'default-start-date')
    options['end_date'] = g.mpuls_config.get_date('search', 'default-end-date')

    # copy options from soptions if included there
    if soptions:
        for name in ["start_date", "end_date", "phase", "type_ending"]:
            if name in soptions:
                options[name] = soptions[name]

    # convert dates to locale
    options['start_date'] = options['start_date']
    options['end_date'] = options['end_date']
    return options


class EvaluateController(BaseController):

    def index(self):
        return self.evaluate()

    def reset_filter(self, id):
        if session.has_key("evaluation_ids"):
            del session['evaluation_ids']
            session.save()
        return self.evaluate(id)

    def evaluate(self, id=None):
        c.evalconfig = EvaluationConfig(get_configfile(id), None, None, None,
                                        None, None, None, None, None)
        evaloptions = get_search_options(session.get('evaluation.options'), id,
                                         session.get("evaluation_ids", ()))

        # If user selects adele-evaluation render page with disabled
        # configuration elements.  Change default params
        if id == '0':
            evaloptions['phase'] = g.mpuls_config.get('evaluations',
                                                      'adele-phases')
            c.adele_evaluation = True
        else:
            c.adele_evaluation = False

        c.phases = PhaseFactory().build()
        defaults = EvaluationFormValidator().from_python(evaloptions)
        form = render('/evaluation/evaluate.mako')
        return formencode.htmlfill.render(form, defaults=defaults,
                                          errors={}, auto_insert_errors=False)

    def _get_evalparams(self, form_result):
        params = {}
        params['id'] = form_result['id']
        params['start_date'] = str(form_result['start_date'])
        params['end_date'] = str(form_result['end_date'])
        params['typelist'] = form_result['typelist']
        params['phase'] = form_result['phase']
        params['type_ending'] = form_result['type_ending']

        where_clauses = []

        # Dates
        # Build timeframes bases on selected phases.
        dates = []
        all_phases = []
        tdates = []
        phase = params.get('phase')
        dates.append('( ')
        if (params.get('start_date') != 'None'
            and params.get('end_date') != 'None'):
            if phase:
                for p in phase:
                    if int(p) >= 0:
                        # phase is unknown -> has no start and endphase
                        sdf, edf = get_phase_dates(p)
                        suc = get_phase_successors(p)
                        all_phases.extend(p)
                        tdates.append("('%s'::date <= %s AND '%s'::date >= %s"
                                      " AND phase IN (%s) )"
                                      % (params.get('start_date'), edf,
                                         params.get('end_date'), sdf,
                                         ",".join(["%s" % s for s in suc])))
                    else:
                        tdates.append('phase IN (-1)')
                dates.append(" OR ".join(tdates))
            else:
                dates.append("FALSE")
        else:
            dates.append("TRUE")
        dates.append(' )')

        where_clauses.append("".join(dates))

        selected_ids = "TRUE"
        if form_result.get("selected_ids"):
            selected_ids = ("(m.id in (%s))"
                            % (",".join(map(str, form_result["selected_ids"]))))

        where_clauses.append(selected_ids)

        where_clauses.append(self.type_ending_clause(params))

        params['sql'] = ("SELECT %%(fields)s from master_tbl_eval_total_view m"
                         " WHERE %s " % (" AND ".join(where_clauses)))


        #Datefields
        if all_phases:
            params['start_date_field'] = get_phase_dates(min(all_phases))[0]
            params['end_date_field'] = get_phase_dates(max(all_phases))[1]
        else:
            params['start_date_field'] = None
            params['end_date_field'] = None
        return params

    def type_ending_clause(self, params):
        """Return the condition to search based on the type of ending.
        Derived classes should override this if necessary as the details
        are usually application specific.
        """
        return "TRUE"

    def evaluateAction(self):
        """Return an HTML file containing the result of one or more evaluations.
        """
        params = formencode.variabledecode.variable_decode(request.params)
        validator = EvaluationFormValidator()

        id = int(params['id'])
        config_file = get_configfile(id)

        # Check values
        try:
            form_result = validator.to_python(params)
        except formencode.Invalid, error:
            log.exception("Error while decoding form parameters")
            form_result = error.value
            form_errors = error.error_dict or {}
            c.phases = PhaseFactory().build()
            c.evalconfig = EvaluationConfig(config_file, None, None, None,
                                            None, None, None, None, None)
            form = render('/evaluation/evaluate.mako')
            return formencode.htmlfill.render(form, defaults=form_result,
                                              errors=form_errors,
                                              auto_insert_errors=False)

        # Build evaluation
        eval_params = self._get_evalparams(form_result)
        try:
            try:
                conn = db.getConnection()
                evalconfig = EvaluationConfig(config_file, conn,
                                              eval_params['start_date'],
                                              eval_params['end_date'],
                                              None, None, None,
                                              eval_params['sql'],
                                              eval_params['typelist'])
                evalset = EvaluationSet(evalconfig)
                evalset.evaluate()
            except:
                log.exception(_('Error: Evaluation failed'))
        finally:
            db.recycleConnection(conn, None)

        exporter = EvaluationExportHTML(show_percent=form_result['show_percent'])
        c.result = evalset.export(exporter)
        session['evaluation.params'] = eval_params
        session['evaluation.options'] = form_result
        session.save()
        return render('/evaluation/result.mako')


    def exportXML(self):
        '''Return an XML file containing the result of one or more evaluations.
        '''
        response.headers['Content-Type'] = 'application/xml; charset=utf8'
        response.headers['Content-Disposition'] = \
                                  'attachment; filename=evaluation-export.xml'
        return self._do_export(EvaluationExportXML)

    def exportCSV(self):
        '''Return an CSV file containing the result of one or more evaluations.
        '''
        response.headers['Content-Type'] = 'application/csv; charset=utf8'
        response.headers['Content-Disposition'] = \
                                  'attachment; filename=evaluation-export.csv'
        return self._do_export(EvaluationExportCSV)

    def _do_export(self, export_class):
        params = session.get('evaluation.params')
        try:
            try:
                conn = db.getConnection()
                cur = None
                evalconfig = EvaluationConfig(get_configfile(params['id']),
                                              conn,
                                              params['start_date'],
                                              params['end_date'],
                                              params['start_date_field'],
                                              params['end_date_field'],
                                              None,
                                              params['sql'],
                                              params['typelist'])
                evalset = EvaluationSet(evalconfig)
                evalset.evaluate()
                return evalset.export(export_class())
            except:
                log.exception("Export with %r failed", export_class)
        finally:
            db.recycleConnection(conn, cur)
