# -*- coding: utf-8 -*-

import logging
from cgi import escape
import datetime
import itertools

import formencode

from pylons import request, c, g, h
from pylons.controllers.util import abort, redirect_to

from formed.instance.backends.postgres import DBFactory as InstanceFactory, \
     DBRepeatInstanceNode as RepeatInstanceNode
from formed.instance.tree import SetDataException, RepeatInstanceContainerNode


from mpulsweb.lib.translation import _
from mpulsweb.lib.security import checkRole
from mpulsweb.lib.db import  PostgresDBInterface
from mpulsweb.lib.navigation import  mark_folderopen

from mpulsweb.lib.base import BaseController, render, session
from mpulsweb.lib.renderer import ViewRenderer, RepeatGroupRenderer, \
     ErrorRenderer, apply_hidden_booleans, get_child_num, icon
from mpulsweb.lib.helpers import url_for, dd_mm_YYYY as format_date, tag
from mpulsweb.model.phase import PhaseFactory, PC_MANUAL, PC_FULLAUTO


log = logging.getLogger(__name__)


def convert_error_items(error_items):
    errors = {}
    defaults = {}
    for items in error_items:
        for e in items:
            errors[e.name] = e.msg
            defaults[e.name] = e.bad
    return errors, defaults


def convert2dic(params):
    result = {}
    for i in params:
        # libformed
        if (i not in ['save', 'savenext', 'page']
            and not i.startswith(('add_rg', 'del_rg'))):
            result[i] = params[i]
    return result

def prepare_htmlfill_defaults(defaults):
    """Prepare default values for htmlfill.

    The defaults parameter should be a dictionary mapping field names to
    values.  The return value is a new dictionary containing the
    converted defaults.

    String values are passed through unchanged.  Date objects (instances
    of datetime.date) are converted using the format_date function.
    Other types are also passed through unchanged and it is assumed that
    the default conversion to strings that htmlfill performs does the
    right thing for these values.
    """
    converted = {}
    for key, value in defaults.iteritems():
        if isinstance(value, datetime.date):
            value = format_date(value)
        converted[key] = value
    return converted

def get_rendered_page(id):
    factory = InstanceFactory(g.formedTree, PostgresDBInterface())
    instance_tree = factory.loadInstanceTreeByIdentifier(
        session['case'].id,
        session.get('uncommited_fields'))

    log.debug('Getting page "%s"' % id)
    page = instance_tree.getItem(id)

    if page:
        page_label = page.getMeta().getDescription()

        # Check if there is help available for this page
        help = []
        if page.getMeta().hasHelp():
            help.append(tag("a", target="_blank",
                           href=url_for(controller="/annotations",
                                        action="help", id=page.getMeta().getName())))
            help.append(tag("img", src=icon("formular/help.png"), alt="help",
                        width=12,
                        height=12))
            help.append('</a>')

        if isinstance(page.getParent(), RepeatInstanceNode):
            page_label += " %s" % get_child_num(page.getParent())
        c.formularheader = ("<h1>%s %s</h1>" % (escape(page_label), "".join(help)))
        if isinstance(page, RepeatInstanceContainerNode):
            renderer = RepeatGroupRenderer(instance_tree, page,
                                           session.get('render_mode',
                                                       'ro') == 'ro',
                                           g.mpuls_config.get("renderer",
                                                              "rg_digests"))
            errors = {}
            defaults = {}
        else:
            c.formularerrors = u"Fehler"
            renderer = ViewRenderer(instance_tree, page,
                                    session.get('render_mode', 'ro') == 'ro')
            errors, defaults = convert_error_items(page.getErrors())
        try:
            mark_folderopen(session['navigation.tree'].getTreeItem(id))
            if session['navigation.tree'].getTreeItem(id).nextKey():
                c.formular = renderer.render(showNext=True)
            else:
                c.formular = renderer.render(showNext=False)
        except Exception, e:
            log.exception(e)
        error_renderer = ErrorRenderer(errors, renderer.warnings,
                                       renderer.notices, renderer.widget_order)
        c.formularerrors = error_renderer.render(page)
        c.page_id = page.getIdentifier()
        session['case'].setPageId(c.page_id)
        session['navigation.selectedpage'] = id
        session.save()
    else:
        log.error('Page not found %s' % id)
        return abort(404)

    # Render form and use htmlfill to fillout the form and mark errorfields.
    form = render('/formularpage/content.mako')
    # Currently values of the fields are set within the renderer.
    # Htmlfill is used to fill in the error values from the error items.
    defaults = prepare_htmlfill_defaults(defaults)

    # Parse the real errors
    parsedForm = formencode.htmlfill.render(form, defaults=defaults,
                                            errors=errors,
                                            auto_insert_errors=False,
                                            force_defaults=False)

    # Replace "form:warnings" with "form:errors" so that htmlfill understands it
    parsedForm = parsedForm.replace('form:warning', 'form:error')

    # make a new dictionary because the warnings have the warning text in a (one
    #element) list
    warningList = {}
    for key, value in itertools.chain(renderer.warnings.iteritems(),
                                      renderer.notices.iteritems()):
        warningList[key] = value[0]

    # Let htmlfill parse the form again
    parsedForm = formencode.htmlfill.render(parsedForm, errors=warningList,
                                            error_class='warning',
                                            auto_insert_errors=False,
                                            force_defaults=False)
    return parsedForm


class FormularpageController(BaseController):

    def index(self):
        # Return a rendered template
        # return render('/formularpage.mako')
        # or, return a response
        return 'Hello World'

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def select(self, id):
        c.formular = get_rendered_page(id)
        return render('/formularpage/formular.mako')

    @checkRole(('cm_ka'))
    def edit(self, id):
        session['render_mode'] = 'rw'
        session.save()
        return self.select(id)

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def show(self, id):
        session['render_mode'] = 'ro'
        session.save()
        return self.select(id)

    @checkRole(('cm_ka'))
    def undo(self, id):
        unc = session.get('uncommited_fields')
        if unc:
            try:
                del unc[id]
            except KeyError:
                pass
            else:
                session['uncommited_fields'] = unc
                session.save()
        return self.select(session['navigation.selectedpage'])

    @checkRole(('cm_ka'))
    def save(self):
        case_id = session['case'].id
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        instance_tree = factory.loadInstanceTreeByIdentifier(
            case_id,
            session.get('uncommited_fields'))
        page_id = request.params['page']

        form_result = convert2dic(request.params)
        apply_hidden_booleans(form_result)

        self._before_set_data_hook(case_id, form_result)

        try:
            instance_tree.setData(form_result)
        except SetDataException, err:
            # We can ignore this exception because errors that occurred
            # will be shown to the user on the form page anyway via the
            # uncommitted fields data stored in the session.
            pass
        session['uncommited_fields'] = instance_tree.commit()
        session.save()

        # How to proceed after saving the data?
        self._handle_repeat_group_redirects()

        if request.params.has_key('savenext'):
            page_id = session['navigation.tree'].getTreeItem(page_id).nextKey().key

        # Check consistence of case and retag
        self._check_case_validity(case_id)

        phase_changed = self._handle_automatic_phase_change(page_id)
        if phase_changed:
            return phase_changed

        return self.select(page_id)

    def _before_set_data_hook(self, case_id, form_result):
        """Modify the form_result dict of the save method before it's applied.
        Override in derived classes to handle application specific
        special cases.  The default implementation does nothing.
        """

    def _check_case_validity(self, case_id):
        """Check validity of the case given by case_id."""
        case = self._loadCase(case_id)
        case.checkValidity()

    def _handle_automatic_phase_change(self, page_id):
        """Change Phase automatically depending on the state of the case"""

        DIALOG_FULLAUTOMATIC_CHANGE_TITLE=_("Phase automatically changed")
        DIALOG_FULLAUTOMATIC_CHANGE_TEXT=_("The phase of the case was changed automatically \
            after checking the filled out required fields. The phase changed from '%s' to '%s'. \
            Please click on 'Ok', to proceed editing the case.")
        DIALOG_HALFAUTOMATIC_CHANGE_TITLE=_("Change phase?")
        DIALOG_HALFAUTOMATIC_CHANGE_TEXT=_("After checking the filled out required fields, it \
            seems that a phase change can be done from phase '%s' to '%s'. Please click on 'Yes' \
            to perform the change. Click on 'No' to perform the change later manually on \
            the phase overview page.")

        changemode = g.mpuls_config.get('phases', 'changemode')
        if changemode != PC_MANUAL:
            phases = PhaseFactory().load(session['case'].id)
            log.debug("current phase: %s" % phases.get_current_phase_id())
            log.debug("max phase? %s" % phases.get_max_phase_id())
            current_phase = phases.get_current_phase_id()
            log.debug(g.mpuls_config.get('phases', 'description'))
            reachable_phase = phases.get_max_phase_id()
            if current_phase != reachable_phase:
                cpd = phases.get_phase(current_phase).getDescription()
                rpd = phases.get_phase(reachable_phase).getDescription()
                if changemode == PC_FULLAUTO:
                    log.debug("fullautomatic phase change!")
                    #load case and set phase
                    case = g.case_factory.loadById(session['case'].id)
                    case.setPhase(reachable_phase)
                    # build dialog
                    c.dialog_title = DIALOG_FULLAUTOMATIC_CHANGE_TITLE
                    c.dialog_text = h.literal((DIALOG_FULLAUTOMATIC_CHANGE_TEXT
                                     % (cpd, rpd)))
                    c.url_ok = url_for(controller="formularpage",
                                       action="select", id=page_id)
                    return render('/phase/dialogs/success_fullautomatic_set_phase.mako')
                else:
                    log.debug("halfautomatic phase change!")
                    # build dialog
                    c.dialog_title = DIALOG_HALFAUTOMATIC_CHANGE_TITLE
                    c.dialog_text = h.literal((DIALOG_HALFAUTOMATIC_CHANGE_TEXT
                                     % (cpd, rpd)))
                    c.url_ok = url_for(controller="phase", action="setPhase",
                                       id=session['case'].id,
                                       value=reachable_phase)
                    c.url_no = url_for(controller="formularpage",
                                       action="select", id=page_id)
                    return render('/phase/dialogs/success_halfautomatic_set_phase.mako')
        return None

    def _handle_repeat_group_redirects(self):
        """Redirect the appropriate repeat group action implied in the request
        """
        for param in request.params:
            if param.startswith('add_rg') and not param.endswith(('x', 'y')):
                if param == 'add_rg':
                    redirect_to(controller='/repeatgroup', action="new",
                                id=request.params.get('add_rg'))
                else:
                    redirect_to(controller='/repeatgroup', action="new",
                                id=param.split('|')[1])
            elif param.startswith('del_rg') and not param.endswith(('x', 'y')):
                redirect_to(controller='/repeatgroup', action="delete",
                            id=param.split('|')[1])


    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def printout(self, id):
        c.print_version = 1
        render_mode = session['render_mode']
        session['render_mode'] = 'ro'
        session.save()
        page = self.select(id)
        session['render_mode'] = render_mode
        session.save()
        return page
