# Copyright 2007, 2008, 2010 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.

"""The base Controller API

Provides the BaseController class for subclassing, and other objects
utilized by Controllers.
"""

import md5
import logging
import locale
from time import time

from webob.exc import HTTPUnauthorized, HTTPNotFound

from routes import url_for

from pylons import c, config, g, request, response
from pylons.controllers import WSGIController
from pylons.controllers.util import redirect_to, redirect
#from mpulsweb.lib.translation import _
#from pylons.templating import render_mako as render
from pylons.templating import cached_template, pylons_globals 

from webhelpers.html import literal

from mpulsweb.config.importer import import_overridable_module
import mpulsweb.lib.db as db
from mpulsweb.lib.translation import set_lang, _, ungettext
from mpulsweb.lib.timelog import timeLog
from mpulsweb.lib.navigation import get_navigation
from mpulsweb.lib.app_globals import session
import mpulsweb.lib.helpers as h
from mpulsweb.lib.security import userIdentity, get_db_parameters

# These names are exported from this module largely for historical
# reasons.  Ideally the module should probably only export
# BaseController and render which are customarily imported from this
# module in Pylons.
__all__ = ["_", "BaseController", "c", "g", "h", "redirect_to", "render",
           "request", "response", "session"]


COOKIE_NOT_FOUND = ("Cookie konnte nicht gefunden werden."
                    " Haben Sie Cookies in Ihrem Browser aktiviert?")
IDENTITY_CHANGED = "Ihre Identitaet hat sich geaendert."

log = logging.getLogger(__name__)


def auth_cookie_name():
    """Return the name of authorization cookie.
    The name is taken from the config setting mpuls.app.auth_cookie_name
    and defaults to 'mpuls_auth'.
    """
    return config.get("mpuls.app.auth_cookie_name", "mpuls_auth")


# Added own render_mako function which addes custom translation functions
def render_mako(template_name, extra_vars=None, cache_key=None, 
                cache_type=None, cache_expire=None):
    """Render a template with Mako
    
    Accepts the cache options ``cache_key``, ``cache_type``, and
    ``cache_expire``.
    
    """    
    # Create a render callable for the cache function
    def render_template():
        # Pull in extra vars if needed
        globs = extra_vars or {}
        
        # Second, get the globals
        globs.update(pylons_globals())

        # Thirs overwritw '_'
        globs.update({'_': _, 'ungettext': ungettext})
        globs["session"] = session

        # Grab a template reference
        template = globs['app_globals'].mako_lookup.get_template(template_name)
        
        return literal(template.render_unicode(**globs))
    
    return cached_template(template_name, render_template, cache_key=cache_key,
                           cache_type=cache_type, cache_expire=cache_expire)
render = render_mako

class BaseController(WSGIController):

    def __init__(self):
        self.db = None
        self.page_cache = None

    def _loadCase(self, id):
        case = g.case_factory.loadById(id)
        session_case = case.getSessionCase()
        if session.has_key('case'):
            old_caseid = session.get('case').id
        else: old_caseid = None

        # New session code
        session['case'] = session_case

        # Only reset navigation if case is switched
        if str(old_caseid) != str(id):
            open_folders = []
            navigation = get_navigation(id)
            session['render_mode'] = 'ro'
            session['navigation.tree'] = navigation
            session['navigation.openfolders'] = open_folders
            session['navigation.selectedpage'] = None 
        session.save()
        return case

    def __before__(self, environ):
        mpuls_session_module = import_overridable_module("lib.session")
        environ['paste.registry'].register(session,
                                           mpuls_session_module.MPulsSession())

        try:
            storedHash = session['AUTH']
            try:
                shared = request.cookies[auth_cookie_name()]
            except KeyError:
                log.error("Cannot find %s cookie -> HTTPUnauthorized",
                          auth_cookie_name())
                raise HTTPUnauthorized(detail=COOKIE_NOT_FOUND)

            if storedHash != md5.new(shared + userIdentity()).digest():
                log.error("Stored hash does't match new hash "
                          "-> HTTPUnauthorized")
                raise HTTPUnauthorized(detail=IDENTITY_CHANGED)
        except KeyError:
            pass

        # Set language for translation based on browser setting. Add a
        # default language based on the server's locale settings.
        deflang = locale.getlocale()[0]
        all_languages = request.languages + [deflang]
        set_lang(all_languages)
        log.debug('Language set to %s', all_languages)

        # Check if the user is already authorized.
        # If not then redirect the user to the login page
        try:
            user = session['USER_AUTHORIZED']
            db_parameters = get_db_parameters(user.login, user.password,
                                              config.get('mpuls.db.host'),
                                              config.get('mpuls.db.port'),
                                              config.get('mpuls.db.database'),
                                              config.get('mpuls.db.user'))

            self.db = db.DB(**db_parameters)
            db.enter(self.db)

            # Is the user activated?
            log.debug('User accound activated: %s' % user.isActivated())
            log.debug('User needs new pass: %s' % user.needsNewPass())
            if (not user.isActivated() and
                request.path_info not in ['/auth/accountDeactivated']):
                redirect_to('/auth/accountDeactivated')

            # Do the user needs to change his passwort?
            elif (user.needsNewPass() and
                  request.path_info not in
                  ['/usersettings/changePasswordOnLogin',
                   '/usersettings/changePasswordOnLoginAction']):
                redirect_to('/usersettings/changePasswordOnLogin')

        except KeyError:
            p = str(request.path_info)
            if not p in ('/auth/login', '/auth/loginAction'):
                redirect(config.get("mpuls.auth.login_url")
                         or url_for(controller='/auth', action='login'))

    def __call__(self, environ, start_response):
        """Invoke the Controller"""
        # WSGIController.__call__ dispatches to the Controller method
        # the request is routed to. This routing information is
        # available in environ['pylons.routes_dict']
        startTime = time()
        try:
            return WSGIController.__call__(self, environ, start_response)
        finally:
            _cache = self.page_cache
            if _cache:
                self.page_cache = None
            _db = self.db
            if _db:
                self.db = None
                db.leave(_db)
                _db.closeConnections()
            stopTime = time()
            timeLog(stopTime - startTime)

    def _checkInt(self, i):
        try:
            return int(i)
        except (TypeError, ValueError):
            self.showError()

    def _checkBool(self, bool):
        confirmed = self._checkInt(bool)
        if confirmed not in [0, 1]:
            self.showError()
        return confirmed == 1

    def showError(self, err=None):
        raise HTTPNotFound()

    def merge_dict(self, leftd, rightd):
        for key in rightd.keys():
            # overwrite key value pairs
            leftd[key] = rightd[key]
        return leftd
