# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

import logging

import psycopg2 as dbapi
import psycopg2.extensions

from paste.registry import StackedObjectProxy

from formed.instance.backends.database import DB as FormEdDBInterface

log = logging.getLogger(__name__)


# Make psycopg2 return unicode objects for strings in query results.
# This is probably not the best way to deal with unicode and the DB-API.
# In particular, it's obviously not directly portable to other DB-API
# implementations.
psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)


db = StackedObjectProxy(name="mpuls.db")


def enter(dbObject):
    db._push_object(dbObject)


def leave(dbObject=None):
    db._pop_object(dbObject)

def quote_connect_parameter(s):
    """Return a quoted version of the string s for with psycopg2.connect."""
    return "'" + s.replace("\\", "\\\\").replace("'", "\\'") + "'"


class DB:

    def __init__(self, database, user, password, host='localhost', port=5432):
        self.database = database
        self.user = user
        self.password = password
        self.host = host
        self.port = int(port)
        self.conn = None
        self.connections = []

    def getConnection(self):
        for used in self.connections:
            if not used[0]:
                used[0] = True
                return used[1]

        # quote all string values for the connect function to make sure
        # spaces, single quotes and backslashes don't lead to problems.
        # This is particularly important for passwords.  The port is an
        # int and os does not need to be quoted.  It seems to be a bug
        # in psycopg2 that it doesn't this kind of quoting automatically
        # when passing the connectin parameters as keyword arguments.
        con = dbapi.connect(database=quote_connect_parameter(self.database),
                            host=quote_connect_parameter(self.host),
                            port=self.port,
                            user=quote_connect_parameter(self.user),
                            password=quote_connect_parameter(self.password))
        used = [True, con]
        self.connections.append(used)
        return con

    def recycleConnection(self, con, cur=None):
        if cur:
            try:
                cur.close()
            except:
                log.exception("Error ignored while closing cursor")
        if con:
            for used in self.connections:
                if used[1] == con:
                    used[0] = False
                    return
            log.warning("WARNING: tried to recycle unused connection")

    def closeConnections(self):
        cons = self.connections
        self.connections = []
        for used in cons:
            if used[0]:
                log.warning("WARNING: Closing an unrecycled connection")
            try:
                used[1].close()
            except:
                log.exception("Error ignored while closing connection")

    def execute(self, statement, *args):
        """Execute an SQL statement in a transaction.

        This method is intended for the common case that a single SQL
        statement has to be executed in a transaction.  This method gets
        a connection with getConnection(), creates a cursor object and
        calls its execute method with the SQL statement and any further
        arguments as arguments.  If the statement is executed
        successfully, the transaction is committed.  If an error occurs
        the transaction is rolled back.  This method does not catch any
        exceptions.

        Any results retrieved from the database are discarded.
        """
        has_commited = False
        cur = None
        con = self.getConnection()
        try:
            cur = con.cursor()
            cur.execute(statement, *args)
            con.commit()
            has_commited = True
        finally:
            if not has_commited and con is not None:
                con.rollback()
            db.recycleConnection(con, cur)


class PostgresDBInterface(FormEdDBInterface):

    def __init__(self):
        FormEdDBInterface.__init__(self)
        self.connection = None

    def acquire_cursor(self):
        if self.connection is None:
            self.connection = db.getConnection()
        return self.connection.cursor()

    def release_cursor(self, cur):
        if self.transaction_level == 0:
            db.recycleConnection(self.connection, cur)
            self.connection = None
        else:
            cur.close()

    def do_rollback(self):
        # if self.connection is still None, no cursor was acquired and
        # there's nothing to do, actually.
        if self.connection is not None:
            self.connection.rollback()

    def do_commit(self):
        # if self.connection is still None, no cursor was acquired and
        # there's nothing to do, actually.
        if self.connection is not None:
            self.connection.commit()


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
