#!/usr/bin/env python

import psycopg2 as dbapi

import sys
import getopt
import traceback

from datetime import timedelta

try:
    import sqlite3 as db
except ImportError:
    from pysqlite2 import dbapi2 as db

from itertools import izip

TAGS_INSERTED = False

SELECT_MASTER = '''SELECT * FROM master_tbl_anonym_view'''

SELECT_MAX_ID = '''SELECT coalesce(max(id), -1) FROM %(relation)s'''

INSERT_STMT = '''INSERT INTO master_tbl (%(fields)s) VALUES (%(placeholders)s)'''

SELECT_FKZ = "SELECT id, database FROM fkz_tbl"

SELECT_STATUS = "SELECT status, zugriff FROM ka_status_tbl WHERE master_id = %(id)s"

INSERT_STATUS = '''
INSERT INTO ka_status_tbl (id, master_id, status, zugriff)
       VALUES (:id, :master_id, :status, :zugriff)'''

SELECT_LOGBOOK = '''
SELECT
    l.datum AS datum,
    md5(l.bearbeiter) AS bearbeiter,
    l.art   AS art,
    l.dauer AS dauer
FROM ka_logbuch_tbl l
WHERE l.master_id = %(master_id)s
'''

INSERT_LOGBOOK = '''
INSERT INTO ka_logbuch_tbl (id, master_id, datum, bearbeiter, art, dauer)
       VALUES (:id, :master_id, :datum, :bearbeiter, :art, :dauer)
'''

SELECT_TAGS = '''
SELECT id, label, description, systemtag FROM tags
'''

INSERT_TAGS = '''
INSERT INTO tags (id, label, description, systemtag)
       VALUES (:id, :label, :description, :systemtag)
'''

SELECT_NM_TAGS_CASES = '''
SELECT tag_id, created FROM nm_tags_cases WHERE master_id = %(id)s
'''

INSERT_NM_TAGS_CASES = '''
INSERT INTO nm_tags_cases (id, master_id, tag_id, created)
       VALUES (:id, :master_id, :tag_id, :created)
'''


ANON_TRANSFER = "SELECT wert = 'on' FROM ka_konfiguration_tbl WHERE bez = 'anon_transfer'"

IN_SELECT  = '''SELECT * FROM %(relation)s_anonym_view WHERE master_id = %%(master_id)s'''
OUT_INSERT = '''INSERT INTO %(relation)s (%(fields)s) VALUES (%(placeholders)s)'''

UPDATE_DATADUMP = "UPDATE dbinfo SET value = CURRENT_DATE WHERE key = 'datadump'"

BLACK_LIST = set([
    'id',
    'uuid_id',
    'master_id'
])

# tables depending on master_tbl
RELATIONS = [
    'rg_ansprechpartner_tbl',
    'rg_elternarbeit_tbl',
    'rg_fehlzeiten_tbl',
    'rg_hilfsmassnahmen_tbl',
    'rg_kontaktdaten_tbl',
    'rg_probleme_tbl',
    'rg_vereinbarungen_tbl'
]


def get_max_id(cur, relation):
    stmt = SELECT_MAX_ID % { 'relation': relation }
    cur.execute(stmt)
    row = cur.fetchone()
    if not row: return -1
    return row[0]

def usage(msg = "", exit_code = 0):
    print >> sys.stderr, """%s
usage:
%s -o <dump.db>
""" % (msg, sys.argv[0])
    sys.exit(exit_code)

def get_fkzs(cur):
    cur.execute(SELECT_FKZ)
    out = []
    while True:
        row = cur.fetchone()
        if not row: break
        out.append(row)
    return out

def copy_relation(in_cur, out_cur, relation, cases):

    orig_select = IN_SELECT % { 'relation' : relation }

    out_counter = get_max_id(out_cur, relation)
    insert_stmnt, fields, row_indices = None, [], []

    for rid, nid in cases:
        in_cur.execute(orig_select, { 'master_id': rid })
        while True:
            row = in_cur.fetchone()
            if not row: break
            out_counter += 1
            values = { 'id': out_counter, 'master_id': nid }
            if insert_stmnt is None:
                # insert statement not prepared yet
                placeholders = []
                for idx, x in enumerate(row):
                    name = in_cur.description[idx][0]
                    if name in BLACK_LIST: continue
                    row_indices.append(idx)
                    placeholders.append(':%s' % name)
                    fields.append(name)
                    values[name] = x
                insert_stmnt = OUT_INSERT % {
                    'relation'    : relation,
                    'fields'      : 'id,master_id,'   + ','.join(fields),
                    'placeholders': ':id,:master_id,' + ','.join(placeholders) }
            else:
                for idx, row_idx in enumerate(row_indices):
                    values[fields[idx]] = row[row_idx]
            out_cur.execute(insert_stmnt, values)

def copy_master(in_cur, out_cur, fkz):

    cases = []

    id_counter = get_max_id(out_cur, 'master_tbl')

    in_cur.execute(SELECT_MASTER)

    stmnt, fields, row_indices, id_idx = None, [], [], None

    while True:
        row = in_cur.fetchone()
        if not row: break

        id_counter += 1
        values = { 'id': id_counter }

        real_id = None

        if stmnt is None:
            placeholders = []
            for idx, x in enumerate(row):
                name = in_cur.description[idx][0]
                if name == 'id': real_id, id_idx = x, idx
                if name in BLACK_LIST: continue
                fields.append(name)
                row_indices.append(idx)
                placeholders.append(':%s' % name)
                values[name] = x

            stmnt = INSERT_STMT % {
                'fields':       "id,"  + ",".join(fields),
                'placeholders': ":id," + ",".join(placeholders) }
        else:
            for idx, row_idx in enumerate(row_indices):
                values[fields[idx]] = row[row_idx]
            real_id = row[id_idx]

        values["fkz"] = fkz[0]

        out_cur.execute(stmnt, values)
        cases.append((real_id, id_counter))

    return cases

def copy_tags(in_cur, out_cur):

    in_cur.execute(SELECT_TAGS)

    while True:
        row = in_cur.fetchone()
        if not row: break
        
        id, label, description, systemtag = row
        out_cur.execute(INSERT_TAGS, {
            'id'            : id, 
            'label'         : label, 
            'description'   : description, 
            'systemtag'     : systemtag
        })
    
def copy_nm_tags_cases(in_cur, out_cur, cases):

    status_counter = get_max_id(out_cur, 'nm_tags_cases')

    for rid, nid in cases:
        in_cur.execute(SELECT_NM_TAGS_CASES, { 'id': rid})
        for row in in_cur.fetchall():
            status_counter += 1
            tag_id, created = row
            out_cur.execute(INSERT_NM_TAGS_CASES, {
                'id'        : status_counter,
                'master_id' : nid,
                'tag_id'    : tag_id, 
                'created'   : created
            })

def copy_status(in_cur, out_cur, cases):

    status_counter = get_max_id(out_cur, 'ka_status_tbl')

    for rid, nid in cases:
        in_cur.execute(SELECT_STATUS, { 'id': rid})
        row = in_cur.fetchone()
        if not row: continue
        status_counter += 1
        status, zugriff = row
        out_cur.execute(INSERT_STATUS, {
            'id'        : status_counter,
            'master_id' : nid,
            'status'    : status,
            'zugriff'   : zugriff
        })

def to_seconds(td):
    if isinstance(td, timedelta):
        days = td.days * 86400
        secs = td.seconds
        ms   = td.microseconds > 500000 and 1 or 0
        return days + secs + ms
    return td

def transfer_logbuch_tbl(in_cur, out_cur, ids):

    new_lb_id = get_max_id(out_cur, 'ka_logbuch_tbl')

    for old_id, new_id in ids:
        in_cur.execute(SELECT_LOGBOOK, { 'master_id': old_id })
        while True:
            row = in_cur.fetchone()
            if not row: break
            datum, bearbeiter, art, dauer = row
            new_lb_id += 1
            out_cur.execute(INSERT_LOGBOOK, {
                'id'        : new_lb_id,
                'master_id' : new_id,
                'datum'     : datum,
                'bearbeiter': bearbeiter,
                'art'       : art,
                'dauer'     : to_seconds(dauer)
            })

def handle(out_cur, fkz, port):

    database = fkz[1]

    in_con, in_cur = None, None
    print >> sys.stderr, "processing '%s'" % database
    try:
        try:
            in_con = dbapi.connect(database = database, port = port)
        except:
            print >> sys.stderr, "Problem with db '%s'" % database
            traceback.print_exc(file = sys.stderr)
            return

        in_cur = in_con.cursor()

        in_cur.execute(ANON_TRANSFER)
        row = in_cur.fetchone()
        if not row or not row[0]:
            print >> sys.stderr, "No transfer allowed by '%s'" % database
            return

        # copy master_tbl
        cases = copy_master(in_cur, out_cur, fkz)

        # copy ka_status_tbl
        copy_status(in_cur, out_cur, cases)

        # copy all depending relations
        for relation in RELATIONS:
            copy_relation(in_cur, out_cur, relation, cases)

        # copy logbook
        transfer_logbuch_tbl(in_cur, out_cur, cases)

        # copy tags
        # Only insert tags one time.
        # FIXME: Tags should be part of the template db.
        #global TAGS_INSERTED
        #if not TAGS_INSERTED:
        #    copy_tags(in_cur, out_cur)
        #    TAGS_INSERTED = True
        copy_nm_tags_cases(in_cur, out_cur, cases)

    finally:
        if in_cur:
            try: in_cur.close()
            except: pass
        if in_con:
            try: in_con.close()
            except: pass

DEFAULT_PORT = 5432

def main():
    outdb = None

    port = DEFAULT_PORT

    try:
        opts, args = getopt.getopt(sys.argv[1:], "o:p:", [ "out=", "port=" ])
    except getopt.GetoptError, err:
        usage(str(err), 1)

    for o, a in opts:
        if o in ("o", "-o", "--out"):
            outdb = a
        elif o in ("p", "-p", "--port"):
            port = int(a)

    if outdb is None: usage("Error: missing out db",   1)

    out_con, out_cur = None, None

    try:
        out_con = db.connect(outdb)
        out_cur = out_con.cursor()

        try:
            fkzs = get_fkzs(out_cur)
            for fkz in fkzs:
                handle(out_cur, fkz, port)

            out_cur.execute(UPDATE_DATADUMP)

            out_con.commit()
        except:
            out_con.rollback()
            raise
    finally:
        if out_cur:
            try: out_cur.close()
            except: pass
        if out_con:
            try: out_con.close()
            except: pass

if __name__ == '__main__':
    main()
