# -*- coding: utf-8 -*-

import sys
import logging
from datetime import datetime, timedelta, date
import traceback

import formencode

from formed.instance.backends.postgres import DBFactory as InstanceFactory
from formed.instance.tree import SetDataException

from mpulsweb.model.logbook import Logbook, LogbookEntry
from mpulsweb.lib.base import BaseController, c, g, render, request, session
from mpulsweb.lib.db import db, PostgresDBInterface
import mpulsweb.lib.helpers as h
from mpulsweb.lib.helper.case import copy_instance_tree, walk_it, copy_field

from waskoweb.lib.validators import \
     ExtraTimeFormValidator1, ExtraTimeFormValidator2, \
     ReopenFormValidator1, ReopenFormValidator1abgel, \
     ReopenFormValidator2, ReopenFormValidator2abgel, \
     RenewFormValidator

log = logging.getLogger(__name__)

REOPENOK_1_HEADER_CONFIRM = u"Fall wiederaufnehmen?"
REOPENOK_1_TEXT_CONFIRM = u"""\
Wollen Sie den Fall wiederaufnehmen? Dabei wird die aktuelle Fallakte
anonymisiert und relevante Teile der Fallakte aus Phase A und Phase B in
eine neue Fallakte kopiert. Sie werden dann in dieser neuen Fallakte
weiterarbeiten."""

REOPENOK_2_HEADER_CONFIRM = u"Wiederaufnahme des jungen Menschen um 3 Monate verlängern?"
REOPENOK_2_TEXT_CONFIRM = u"""\
Wollen Sie die Wiederaufnahme des jungen Menschen um 3 Monate verlängern? Die Fallakte steht danach weiter zur Bearbeitung."""
REOPENNOTOK_HEADER_CONFIRM = u"Nicht genehmigte Wiederaufnahme dokumentieren?"
REOPENNOTOK_TEXT_CONFIRM = u"""\
Sie haben angegeben, dass keine Genehmigung für die Wiederaufnahme
vorliegt! Wenn Sie fortfahren, wird keine weitere Wiederaufnahme,
Verlängerung oder Neuaufnahme mehr möglich sein."""

REOPENOK_1_SUCCESS_HEADER = u"Fallakte erfolgreich wiederaufgenommen."
REOPENOK_1_SUCCESS_TEXT = u'''\
Die Fallakte wurde erfolgreich wiederaufgenommen. Klicken Sie auf "OK",
um die wiederaufgenommene Fallakte zu öffnen.'''

REOPENOK_2_SUCCESS_HEADER = u"Fallakte erfolgreich wiederaufgenommen."
REOPENOK_2_SUCCESS_TEXT = u'''\
Die Fallakte wurde erfolgreich wiederaufgenommen. Klicken Sie auf "OK",
um zur "Eingangsdokumentation" der Fallakte zu gelangen.'''

REOPENNOTOK_SUCCESS_HEADER = u"Nicht genehmigte Wiederaufnahme dokumentiert."
REOPENNOTOK_SUCCESS_TEXT = u'''\
Die nicht genehmigte Wiederaufnahme wurde dokumentiert. Klicken Sie auf
"OK", um zur "Eingangsdokumentation" der Fallakte zu gelangen.'''

RENEW_2_HEADER_CONFIRM = u"Nicht genehmigte Neuaufnahme dokumentieren?"
RENEW_2_TEXT_CONFIRM = u"""\
Sie haben angegeben, das keine Genehmigung für die Neuaufnahme vorliegt!
Wenn Sie fortfahren, wird für diese Fallakte keine erneute Neuaufnahme
mehr möglich sein."""

RENEW_2_SUCCESS_HEADER = u"Nicht genehmigte Neuaufnahme dokumentiert."
RENEW_2_SUCCESS_TEXT = u'''\
Die nicht genehmigte Neuaufnahme wurde dokumentiert. Klicken Sie auf
"OK", um zur "Eingangsdokumentation" der Fallakte zu gelangen.'''

RENEW_HEADER_CONFIRM = u"Fall neu aufnehmen?"
RENEW_TEXT_CONFIRM = u"""\
Wollen Sie den Fall wirklich neu aufnehmen? Dabei wird die aktuelle
Fallakte anonymisiert und relevante Teile der Fallakte aus Phase A und
Phase B in eine neue Fallakte kopiert. Sie werden dann in dieser neuen
Fallakte weiterarbeiten."""

RENEW_SUCCESS_HEADER = u"Fallakte neu aufgenommen."
RENEW_SUCCESS_TEXT = u'''\
Die Fallakte wurde erfolgreich neu aufgenommen. Klicken Sie auf "OK", um
die neue Fallakte zu öffnen. Bitte überprüfen Sie die Angaben auf
Richtigkeit und aktualisieren Sie ggf. die Daten. Die alte Fallakte
wurde der Administration zur Anonymisierung vorgelegt.'''

ABORT_SUCCESS_HEADER = u'Eingangsdokumentation abgebrochen.'
ABORT_SUCCESS_TEXT = u'''\
Die Eingangsdokumentation wurde erfolgreich abgebrochen. Die Fallakte
wurde der Administration zur Löschung vorgelegt. Klicken Sie auf "OK",
um zu Fallaktenübersicht zu gelangen.'''

ABORT_TEXT_CONFIRM = u'''\
Wollen Sie wirklich die Eingangsdokumentation abbrechen und die
Fallakte zum Löschen freigeben? Die Fallakte steht Ihnen danach nicht
mehr zur Bearbeitung zur Verfügung und wird der Administration zum
Löschen vorgelegt.'''
ABORT_HEADER_CONFIRM = u'Eingangsdokumentation abbrechen?'

# items not to be copied from old case
COPY_BLACK_LIST = frozenset(['erstgespraech', 'datum_cm_start'])


def toErrorDicts(e):
    values, errors = {}, {}
    for key, err in e.errors.iteritems():
        key = key.split(':')[0]
        for e in err:
            values[key] = e.bad
            errors[key] = e.msg
    return values, errors

def create_logbook(id=None, header=None, text=None):
    if id is None:
        id = session['case'].id
    logbook = Logbook(id)
    entry = LogbookEntry()
    entry.datum = datetime.now()
    entry.art = 0
    entry.kurz_notiz = header
    entry.notiz = text
    logbook.addEntry(entry)
    logbook.store(session['USER_AUTHORIZED'].id)

def mark_case(id, mark=None):
    data = {}
    if mark:
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(id)
        item =  it.getItem('fn:%s' % id)
        fn = item.getValue()
        if fn:
            fn += '_%s' % mark 
            data['fn:%s' % id] = fn
        else:
            data['fn:%s' % id] = '_%s' % mark 
        it.setData(data)
        it.commit()


def doc_abschluss(id, to_id=None):
    if to_id is None: to_id = id
    data = {}
    factory = InstanceFactory(g.formedTree, PostgresDBInterface())
    oit = factory.loadInstanceTreeByIdentifier(id)
    nit = factory.loadInstanceTreeByIdentifier(to_id)

    data['cm_dok_end_art:%s' % to_id]          = str(_get_value(oit, 'cm_end_art', id))
    data['cm_dok_end_verm:%s' % to_id]         = str(_get_value(oit, 'cm_end_verm', id))
    data['cm_dok_end_sonst:%s' % to_id]        = str(_get_value(oit, 'cm_end_sonst', id))
    data['cm_dok_datum_beendigung:%s' % to_id] = h.dd_mm_YYYY(_get_value(oit, 'datum_cm_ende', id))

    nit.setData(data)
    nit.commit()
    log.debug(data)

def save_formdata(id, data):
    # Füge alte params aus dem Dialog in der verwaltung hinzu und
    # schreibe alles weg
    id = int(id)
    factory = InstanceFactory(g.formedTree, PostgresDBInterface())
    it = factory.loadInstanceTreeByIdentifier(id)
    sparams = session.get('lifetimeparams', {})
    cm2c_t = _get_value(it, 'cm2c_t', id)
    #if cm2c_t:
    #    data['cm2c_t'] = cm2c_t
    #    data['cm2c_t'] += "\r\n\r\n" + sparams.get('anmerkungen', '')
    #else:
    #    data['cm2c_t'] = sparams.get('anmerkungen', '')
    #del sparams['anmerkungen']
    data.update(sparams)
    _set_data(it, data, id)

def _convert4Formed(data, id):
    convertet_data = {}
    for key, value in data.iteritems():
        if key == 'cm2c_t2': continue
        if isinstance(value, date):
            value = h.format_date(value)
        convertet_data[":".join([str(key), str(id)])] = value
    return convertet_data

def _set_data(it, data, case_id, do_write=True):
    convertet_data = _convert4Formed(data, case_id)
    it.setData(convertet_data)
    if do_write:
        it.commit()

def _get_value(it, id, case_id):
    real_id = "%s:%s" % (id, case_id)
    item = it.getItem(real_id)
    if item:
        value = item.getValue()
        log.debug('%s = "%r"' % (real_id, value))
        return value
    log.error('Can not find item "%s"' % real_id)

def checkcase(it, id):

    # Check if case is anonymizable
    case = g.case_factory.loadById(id)
    try:
        case.check_anonymizeability()
        c.is_anonymizable = True
    except:
        c.is_anonymizable = False 

    case_id = id
    D = _get_value

    end_art = D(it, 'cm_end_art', case_id)
    c.phase = D(it, 'phase', case_id)
    c.cm_neuauf_genehm = D(it, 'cm_neuauf_genehm', case_id)

    ## Welche Beendigung?
    c.erfolgreiche_reintegration = False
    c.sonstige_beendigung = False
    if end_art == 1:
        c.erfolgreiche_reintegration = True
    elif end_art == 2:
        c.sonstige_beendigung = True

    # Wieviele Wiederholungen?
    log.debug('Wiederaufnahme1: %s' % D(it, 'cm_wiederauf_genehm_1', case_id))
    log.debug('Wiederaufnahme2: %s' % D(it, 'cm_wiederauf_genehm_2', case_id))
    if D(it, 'cm_wiederauf_genehm_2', case_id) in (1,0):
        c.num_wiederaufnahme = 2
    elif D(it, 'cm_wiederauf_genehm_1', case_id) in (1,):
        c.num_wiederaufnahme = 1
    else:
        c.num_wiederaufnahme = 0

    # Wieviele Verlängerungen?
    if D(it, 'cm_datum_verlaengerung_2', case_id):
        c.num_verlaengerung = 2
    elif D(it, 'cm_datum_verlaengerung_1', case_id):
        c.num_verlaengerung = 1
    else:
        c.num_verlaengerung = 0
    log.debug('Anzahl Verlaengerugen: %s' % c.num_verlaengerung)


    # Wurde Wiederaufnahme abgelehnt? -> alle Optionen ausblenden
    if (D(it, 'cm_wiederauf_genehm_1', case_id) in (0,)
        or D(it, 'cm_wiederauf_genehm_2', case_id) in (0,)):
        c.disable_all = True
        c.wiederaufn_abg = True
    else:
        c.disable_all = False
        c.wiederaufn_abg = False


    # Verlängerung anzeigen
    if (not c.disable_all and c.phase in (2,) and c.num_verlaengerung < 2
        and c.num_wiederaufnahme < 1):
        c.show_verlaengerung = True
    else:
        c.show_verlaengerung = False

    # Wiedervorlage anzeigen
    if (not c.disable_all
        and (c.phase in (3,4,5) 
             and c.erfolgreiche_reintegration
             and c.num_wiederaufnahme < 1
             and c.is_anonymizable)
        or (c.num_wiederaufnahme == 1 and c.phase not in (-1, 0, 1, 3, 4, 5))):
        c.show_wiedervorlage = True
    else:
        c.show_wiedervorlage = False

    # Neuaufnahme anzeigen
    if (not c.disable_all and (c.phase in (3, 4, 5)) and c.sonstige_beendigung
        and (c.num_wiederaufnahme < 1) and (c.cm_neuauf_genehm not in (0, 1))
        and c.is_anonymizable):
        c.show_neuaufnahme = True
    else:
        c.show_neuaufnahme = False

    # Abbruch anzeigen
    if not c.disable_all and c.phase in (-1, 0, 1):
        c.show_abort = True
    else:
        c.show_abort = False


class CaselifetimeController(BaseController):

    def index(self, id=None):
        c.form_errors = {}
        c.form_result = {}
        c.form_defaults= {}
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(id)
        case_id = it.getRootNode().getIdentifier()
        checkcase(it, case_id)
        D = _get_value

        # Verlängerungen
        ende_verl1 = D(it, 'cm_datum_verlaengerung_1', case_id)
        ende_verl2 = D(it, 'cm_datum_verlaengerung_2', case_id)
        if c.num_verlaengerung < 1:
            begin_cm = D(it, 'datum_cm_start', case_id)
            if begin_cm and not ende_verl1:
                ende_verl1 = begin_cm+timedelta(days=458)
        if c.num_verlaengerung == 1 and not ende_verl2:
                ende_verl2 = ende_verl1+timedelta(days=31*3)

        c.form_defaults['datum_cm_ende'] = h.dd_mm_YYYY(D(it, 'datum_cm_ende', case_id))
        c.form_defaults['cm_datum_verlaengerung_1'] = h.dd_mm_YYYY(ende_verl1)
        c.form_defaults['cm_bestaet_verlaengerung_1'] = D(it, 'cm_bestaet_verlaengerung_1', case_id)
        c.form_defaults['cm_anmerk_verlaengerung_1'] = D(it, 'cm_anmerk_verlaengerung_1', case_id)
        c.form_defaults['cm_datum_verlaengerung_2'] = h.dd_mm_YYYY(ende_verl2)
        c.form_defaults['cm_bestaet_verlaengerung_2'] = D(it, 'cm_bestaet_verlaengerung_2', case_id)
        c.form_defaults['cm_anmerk_verlaengerung_2'] = D(it, 'cm_anmerk_verlaengerung_2', case_id)

        # Wiederaufnahme
        wieder1_date = D(it, 'cm_wiederauf_datum_bis_1', case_id)
        wieder2_date = D(it, 'cm_wiederauf_datum_bis_2', case_id)
        if c.num_wiederaufnahme >= 1:
            if wieder1_date and not wieder2_date:
                wieder2_date = wieder1_date+timedelta(days=31*3)

        c.form_defaults['cm_wiederauf_genehm_1'] = D(it, 'cm_wiederauf_genehm_1', case_id)
        c.form_defaults['cm_wiederauf_genehm_datum_1'] = h.dd_mm_YYYY(D(it, 'cm_wiederauf_genehm_datum_1', case_id))
        c.form_defaults['cm_wiederauf_datum_bis_1'] = h.dd_mm_YYYY(wieder1_date) 
        c.form_defaults['cm_wiederauf_genehm_2'] = D(it, 'cm_wiederauf_genehm_2', case_id)
        c.form_defaults['cm_wiederauf_datum_bis_2'] = h.dd_mm_YYYY(wieder2_date)
        c.form_defaults['cm2c_t'] = D(it, 'cm2c_t', case_id)

        form = render('/caselifetime/overview.mako')
        return formencode.htmlfill.render(form,
                                          defaults=c.form_defaults)


    def changeLifetime(self):
        if request.params.has_key('wiederaufnahme'):
            return self.reopenAction()
        elif request.params.has_key('verlaengerung'):
            return self.extraTimeAction()
        elif request.params.has_key('neuaufnahme'):
            return self.renewAction()
        elif request.params.has_key('abort'):
            return self.abortAction(session['case'].id)
        else:
            pass
        return render('/caselifetime/overview.mako')

    def abortcase(self, id, confirmed='0'):
        if confirmed == '1':
            # mark case for deletion
            case = g.case_factory.loadById(id)
            case.setState(3)
            c.success_for = ABORT_SUCCESS_HEADER
            c.success_text = ABORT_SUCCESS_TEXT
            c.url_ok = h.url_for(controller="/case_overview")
            return render('caselifetime/dialogs/success_reopen.mako')
        else:
            c.confirm_for = ABORT_HEADER_CONFIRM
            c.question = ABORT_TEXT_CONFIRM
            c.url_yes = "/caselifetime/abortcase/%s/1" % id
            c.url_no = "/caselifetime/index/%s" % id
            return render('caselifetime/dialogs/confirm_reopen.mako')

    def abortAction(self, id):
        return self.abortcase(id)
        return self.index(id)

    def extraTimeAction(self):

        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(session['case'].id)
        case_id = it.getRootNode().getIdentifier()
        S = _set_data
        checkcase(it, case_id)

        form_result = {}
        form_errors = {}

        if c.num_verlaengerung < 1:
            validator = ExtraTimeFormValidator1()
        else:
            validator = ExtraTimeFormValidator2()

        try:
            # 1. Check. Formencode check general format of date and if value is
            # present
            form_result = validator.to_python(request.params)
            try:
                # Save method will also do semantic check (dates or in
                # chronology correct order) 
                S(it, form_result, case_id)
                create_logbook(header='Verlängerung des Integrationsprozess',
                               text=('Für den Fall wurde eine Verlängerung'
                                     ' des Integrationsprozess dokumentiert.'))
                return self.index(session['case'].id)
            except SetDataException, err:
                form_result_1, form_errors = toErrorDicts(err)
        except formencode.Invalid, error:
            form_result = error.value
            form_errors = error.error_dict or {}

        form_errors = form_errors
        form_result = form_result
        form = render('/caselifetime/overview.mako')
        return formencode.htmlfill.render(form, 
                                          defaults=form_result,
                                          errors=form_errors) 

    def renew_ok(self, id, confirmed='0'):
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        if confirmed == '1':
            data = {}
            new_case = None
            try:
                it = factory.loadInstanceTreeByIdentifier(id)
                # Lege neuen Fall an, es sollen aber nur bestimmte Felder kopiert werden
                fields = []
                for section in ['ee', 'phase-a', 'group-5']:
                    node = it.getItem("%s:%s" % (section, id))
                    for field in walk_it(node):
                        name = field.getIdentifier().split(':')[0]
                        if name not in COPY_BLACK_LIST:
                            fields.append(name)
                        else:
                            log.info('Not copying %s as it is in COPY_BLACK_LIST' % name)
                new_it = copy_instance_tree(it,fields)
                new_id = new_it.getRootNode().getIdentifier()
                data['phase'] = '-1'
                data['cm2c'] = '1'
                doc_abschluss(id, new_id)
                # Dokumentiere Neuaufnahme in neuem Fall 
                save_formdata(new_id, data)
                mark_case(new_id, 'NA')
                create_logbook(id=new_id, header='Neuaufnahme',
                               text=('Für den Fall wurde eine Neuaufnahme'
                                     ' des Integrationsprozess dokumentiert.'))

                # markiere den alten Fall zur Anonymisierung
                old_case = g.case_factory.loadById(id)
                old_case.setState(4)
                mark_case(old_case.id, 'NAU')
                create_logbook(header='Neuaufnahme',
                               text=('Für den Fall wurde eine Neuaufnahme'
                                     ' des Integrationsprozess dokumentiert.'))
            except Exception, err:
                traceback.print_exc(file = sys.stderr)

            c.success_for = RENEW_SUCCESS_HEADER
            c.success_text = RENEW_SUCCESS_TEXT
            c.url_ok = h.url_for(controller="/case/select/%s" % new_id)
            return render('caselifetime/dialogs/success_reopen.mako')
        else:
            c.confirm_for = RENEW_HEADER_CONFIRM
            c.question = RENEW_TEXT_CONFIRM
            c.url_yes = "/caselifetime/renew_ok/%s/1" % id
            c.url_no = "/caselifetime/index/%s" % id
            return render('caselifetime/dialogs/confirm_reopen.mako')

    def renew_notok(self, id, confirmed='0'):
        if confirmed == '1':
            data = {}
            try:
                save_formdata(id, data)
                create_logbook(id=id, header='Neuaufnahme',
                               text=('Für den Fall wurde eine nicht'
                                     ' genehmigte Neuaufnahme des'
                                     ' Integrationsprozess dokumentiert.'))

            except Exception, err:
                log.exception('Error in renew_notok')

            c.success_for = RENEW_2_SUCCESS_HEADER
            c.success_text = RENEW_2_SUCCESS_TEXT
            c.url_ok = h.url_for(controller="formularpage",
                                 action="select", 
                                 id="page-0:%s" % id)
            return render('caselifetime/dialogs/success_reopen.mako')
        else:
            c.confirm_for = RENEW_2_HEADER_CONFIRM
            c.question = RENEW_2_TEXT_CONFIRM
            c.url_yes = "/caselifetime/renew_notok/%s/1" % id
            c.url_no = "/caselifetime/index/%s" % id
            return render('caselifetime/dialogs/confirm_reopen.mako')

    def reopen_ok(self, id, confirmed='0'):
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(id)
        checkcase(it, id)
        if confirmed == '1':
            try:
                data = session['lifetimeparams'] 
                if c.num_wiederaufnahme < 1:
                    ## Dokumentiere Daten im alten Fall!
                    #save_formdata(id, data)
                    ## Reload case after saving data
                    #it = factory.loadInstanceTreeByIdentifier(id)

                    # Lege neuen Fall an, es sollen aber nur bestimmte Felder kopiert werden
                    fields = []
                    for section in ['ee', 'phase-a', 'group-5', 'group-18', 'group-27']:
                        node = it.getItem("%s:%s" % (section, id))
                        for field in walk_it(node):
                            name = field.getIdentifier().split(':')[0]
                            if name not in COPY_BLACK_LIST:
                                fields.append(name)
                            else:
                                log.info('Not copying %s as it is in COPY_BLACK_LIST' % name)
                    new_it = copy_instance_tree(it,fields)
                    new_id = new_it.getRootNode().getIdentifier()
                    
                    doc_abschluss(id, new_id)
                    # Copy Datum der Wiederaufnahme in Erstgespräch und Beginn des CM 
                    data['datum_cm_start'] = data['cm_wiederauf_genehm_datum_1']
                    data['erstgespraech'] = data['cm_wiederauf_genehm_datum_1']
                    # delete 'datum_cm_ende' as it must not be written in the new case
                    del data['datum_cm_ende']
                    data = _convert4Formed(data, new_id)
                    data['phase:%s' % new_id] = '2'
                    data['cm2c:%s' % new_id] = '1'

                    new_it.setData(data)
                    new_it.commit()

                    mark_case(new_id, 'WA')
                    create_logbook(id=new_id, header='Wiederaufnahme',
                                   text=('Für den Fall wurde eine genehmigte'
                                         ' Wiederaufnahme des'
                                         ' Integrationsprozess dokumentiert.'))

                    ## Markiere alten Fall zur Anonymisierung
                    old_case = g.case_factory.loadById(id)
                    old_case.setState(4)
                    mark_case(old_case.id, 'WAU')
                    create_logbook(id=id, header='Wiederaufnahme',
                                   text=('Für den Fall eine Wiederaufnahme des'
                                         ' Integrationsprozess in einer neuen'
                                         ' Fallakte durchgeführt.'))
                else:


                    data['cm2c_t'] += "\n" + data['cm2c_t2']
                    del data['cm2c_t2']
                    save_formdata(id, data)
                    create_logbook(header='Verlängerung Wiederaufnahme',
                                   text=('Für den Fall wurde eine genehmigte'
                                         ' Verlängerung der Wiederaufnahme des'
                                         ' Integrationsprozess dokumentiert.'))
            except Exception, err:
                log.exception(err.__dict__)
            if c.num_wiederaufnahme < 1:
                id = new_it.getRootNode().getIdentifier()
                c.success_for = REOPENOK_1_SUCCESS_HEADER
                c.success_text = REOPENOK_1_SUCCESS_TEXT
                c.url_ok = h.url_for(controller="/case", 
                                     action="select",
                                     id="%s" % id)
            else:
                c.success_for = REOPENOK_2_SUCCESS_HEADER
                c.success_text = REOPENOK_2_SUCCESS_TEXT
                c.url_ok = h.url_for(controller="/formularpage", 
                                     action="select",
                                     id="page-0:%s" % id)
            return render('caselifetime/dialogs/success_reopen.mako')
        else:
            if c.num_wiederaufnahme < 1:
                c.confirm_for = REOPENOK_1_HEADER_CONFIRM
                c.question = REOPENOK_1_TEXT_CONFIRM
                c.url_yes = "/caselifetime/reopen_ok/%s/1" % id
                c.url_no = "/caselifetime/index/%s" % id
            else:
                c.confirm_for = REOPENOK_2_HEADER_CONFIRM
                c.question = REOPENOK_2_TEXT_CONFIRM
                c.url_yes = "/caselifetime/reopen_ok/%s/1" % id
                c.url_no = "/caselifetime/index/%s" %id
            return render('caselifetime/dialogs/confirm_reopen.mako')

    def reopen_notok(self, id, confirmed='0'):
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(session['case'].id)
        checkcase(it, id)
        data = {}
        if confirmed == '1':
            try:
                # Füge alte params aus dem Dialog in der verwaltung
                # hinzu und schreibe alles weg
                save_formdata(id, data)
                create_logbook(header='Wiederaufnahme',
                               text=('Für den Fall wurde eine nicht genehmigte'
                                     ' Wiederaufnahme des Integrationsprozess'
                                     ' dokumentiert.'))
            except Exception, err:
                print err

            c.success_for = REOPENNOTOK_SUCCESS_HEADER
            c.success_text = REOPENNOTOK_SUCCESS_TEXT
            c.url_ok = h.url_for(controller="/formularpage",
                                 action="select",
                                 id="page-0:%s" % id)
            return render('caselifetime/dialogs/success_reopen.mako')
            return self.index()
        else:
            c.confirm_for = REOPENNOTOK_HEADER_CONFIRM
            c.question = REOPENNOTOK_TEXT_CONFIRM
            c.url_yes = "/caselifetime/reopen_notok/%s/1" % id
            c.url_no = "/caselifetime/index/%s" % id
            return render('caselifetime/dialogs/confirm_reopen.mako')

    def reopenAction(self):
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(session['case'].id)
        case_id = it.getRootNode().getIdentifier()
        checkcase(it, case_id)
        S = _set_data
        form_result = {}
        form_errors = {}

        if c.num_wiederaufnahme < 1:
            if request.params.get('cm_wiederauf_genehm_1') == '0':
                validator = ReopenFormValidator1abgel()
            else:
                validator = ReopenFormValidator1()
        else:
            if request.params.get('cm_wiederauf_genehm_2') == '0':
                validator = ReopenFormValidator2abgel()
            else:
                validator = ReopenFormValidator2()

        try:
            form_result = validator.to_python(request.params)
            session['lifetimeparams'] = form_result
            session.save()
            try:
                # Lets try to insert some dates (do not actually insert them)
                # and check if they match all rules.
                S(it, form_result, case_id, do_write=False)
                if (form_result.get('cm_wiederauf_genehm_1') == '1'
                    or form_result.get('cm_wiederauf_genehm_2') == '1'):
                    return self.reopen_ok(session['case'].id)
                else:
                    return self.reopen_notok(session['case'].id)
            except SetDataException, err:
                form_result_1, form_errors = toErrorDicts(err)
        except formencode.Invalid, error:
            #TODO: use htmlfill
            form_result = error.value
            form_errors = error.error_dict or {}

        form = render('/caselifetime/overview.mako')
        return formencode.htmlfill.render(form, 
                                          defaults=form_result,
                                          errors=form_errors)

    def renewAction(self):

        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(session['case'].id)
        case_id = it.getRootNode().getIdentifier()
        S = _set_data
        checkcase(it, case_id)

        validator = RenewFormValidator()
        form_result = {}
        form_errors = {}

        try:
            form_result = validator.to_python(request.params)
            session['lifetimeparams'] = form_result
            session.save()
            try:
                S(it, form_result, case_id, do_write = False)
                if form_result.get('cm_neuauf_genehm') == '1':
                    return self.renew_ok(case_id)
                else:
                    return self.renew_notok(case_id)
            except SetDataException, err:
                form_result_1, form_errors = toErrorDicts(err)

        except formencode.Invalid, error:
            form_result = error.value
            form_errors = error.error_dict or {}

        form_result = form_result
        form_errors = form_errors
        form = render('/caselifetime/overview.mako')
        return formencode.htmlfill.render(form,
                                          defaults=form_result,
                                          errors=form_errors)

