# -*- coding: utf-8 -*-
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
#

import logging
from datetime import datetime, timedelta

import mpulsweb.lib.helpers as h
from mpulsweb.lib.base import session, g
from mpulsweb.lib.db import db
from mpulsweb.model.case import MpulsCaseOverview
from mpulsweb.model.appointment import AppointmentOverview, CaseAppointmentFactory
from mpulsweb.controllers.case_overview import default_phases


# Get cases with long extensions only for working status.
GET_MAX_CM_CASES_NO_EXTENSIONS = """
SELECT master.id,
    ABS(extract(day from (datum_cm_start - coalesce(datum_cm_ende,
            now())))) AS dauer
FROM master_tbl_view master JOIN ka_status_tbl_view status
ON master.id = status.master_id
WHERE
    status.status = 1
    AND ABS(extract(day from (datum_cm_start - coalesce(datum_cm_ende,
            now())))) > 365
    AND cm_datum_verlaengerung_1 IS NULL
    AND cm_datum_verlaengerung_2 IS NULL;
"""

GET_MAX_CM_CASES_1_EXTENSION = """
SELECT master.id,
    ABS(extract(day from (datum_cm_start - coalesce(cm_datum_verlaengerung_1,
            now())))) AS dauer
FROM master_tbl_view master JOIN ka_status_tbl_view status
ON master.id = status.master_id
WHERE
    status.status = 1
    AND ABS(extract(day from (datum_cm_start - coalesce(cm_datum_verlaengerung_1,
            now())))) > 31*15
    AND cm_datum_verlaengerung_1 IS NOT NULL
    AND cm_datum_verlaengerung_2 IS NULL;
"""

GET_MAX_CM_CASES_2_EXTENSIONS = """
SELECT master.id,
    ABS(extract(day from (datum_cm_start - coalesce(cm_datum_verlaengerung_2,
            now())))) AS dauer
FROM master_tbl_view master JOIN ka_status_tbl_view status
ON master.id = status.master_id
WHERE
    status.status = 1
    AND ABS(extract(day from (datum_cm_start - coalesce(cm_datum_verlaengerung_2,
            now())))) > 31*18
    AND cm_datum_verlaengerung_1 IS NOT NULL
    AND cm_datum_verlaengerung_2 IS NOT NULL;
"""

log = logging.getLogger(__name__)



class CaseReminderMaxLength(AppointmentOverview):

    def __init__(self):
        self.appointment_list = []
        cases = MpulsCaseOverview()
        user = session['USER_AUTHORIZED']
        
        # Cases with 2 extensions, 18 months maximum
        cases_2_extensions = self.get_cases(GET_MAX_CM_CASES_2_EXTENSIONS)
        max_duration = 18*31
        for case_id in cases_2_extensions:
            duration = cases_2_extensions[case_id].get('dauer')
            self.appointment_list.append(self.make_extension_appointment(case_id,
                                                        duration, max_duration))
            
        # Cases with 1 extension, 15 months maximum
        cases_1_extension = self.get_cases(GET_MAX_CM_CASES_1_EXTENSION)
        max_duration = 15*31
        for case_id in cases_1_extension:
            duration = cases_1_extension[case_id].get('dauer')
            self.appointment_list.append(self.make_extension_appointment(case_id,
                                                        duration, max_duration))
        
        # Cases with no extensions, 12 months maximum
        cases_no_extensions = self.get_cases(GET_MAX_CM_CASES_NO_EXTENSIONS)
        max_duration = 365
        for case_id in cases_no_extensions:
            duration = cases_no_extensions[case_id].get('dauer')
            self.appointment_list.append(self.make_extension_appointment(case_id,
                                                        duration, max_duration))
        
    
    def get_cases(self, sql):
        conn, cur = None, None
        list = {}
        try:
            conn = db.getConnection()
            cur = conn.cursor()
            cur.execute(sql)
            
            result = cur.fetchall()
            for r in result:
                list[r[0]] = {'dauer': r[1]}
            return list
        finally:
            db.recycleConnection(conn, cur)
        return False

    def make_extension_appointment(self, case_id, duration, max_duration):
        factory = CaseAppointmentFactory()
        type = 1
        case = g.case_factory.loadById(case_id)
        title = u"Zu langer Case-Management der Fallakte von %s, %s" %\
                                            (case.last_name, case.first_name)
        
        description = u''' Die Fallakte überschreitet mit %.f Tagen die maximale
                Dauer der Phase Case-Management von %s Tagen.''' % \
                (duration, max_duration)
        appointment = factory.createByData(None, title, datetime.now(), None,
                                            description, case.id, type)
        return appointment

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8:
